/*
 * This is the echoip-slatecave templating engine.
 * It wraps around tera in is specialized for echoip-slatecave.
 */

use axum::{
	body::{Bytes,Full},
	headers::HeaderValue,
	http::StatusCode,
	http::header,
	http::header::SET_COOKIE,
	response::IntoResponse,
	response::Response,
};
use axum_extra::extract::cookie::Cookie;
use axum_extra::extract::cookie;
use tera::Tera;
use toml::Table;

use crate::view::View;
use crate::mycelium::MycView;
use crate::mycelium::MycFormat;
use crate::mycelium::MycFormatFamily;
use crate::mycelium::MycQuerySettings;
use crate::settings::QuerySettings;

/* The engine itself */

#[derive(Clone)]
pub struct Engine {
	pub tera: Tera,
	pub template_config: Option<Table>,
}

impl Engine {
	pub async fn render_view(
		&self,
		settings: &QuerySettings,
		view: View,
	) -> Response {
		let status_code = view.get_status_code(&settings);

		let mut response = match settings.format.get_family() {
			MycFormatFamily::Template => {
				let template_name = view.get_template_name();
				let format = settings.get_format();
				let mime_type = format.get_mime_type();
		
				let mut context = tera::Context::new();
				context.insert("view", &template_name);
				//intented for shared macros
				context.insert("format", &format.get_name());
				context.insert("mimetype", &mime_type.to_string());
				context.insert("data", &view);
				context.insert("extra", &self.template_config);
				settings.initalize_template_context(&mut context);

				match self.tera.render(&(template_name.clone()+&format.get_file_extension()), &context) {
					Ok(text) => {
						let mut response = (
							[(
								header::CONTENT_TYPE,
								HeaderValue::from_str(mime_type.as_ref())
									.expect("MimeType should always be a valid header value.")
							)],
							Into::<Full<Bytes>>::into(text),
						).into_response();
						view.update_response(&mut response, &settings);
						response
					},
					Err(e) => {
						println!("There was an error while rendering template {}: {e:?}", template_name);
						(
							StatusCode::INTERNAL_SERVER_ERROR,
						format!("Template error in {}, contact owner or see logs.\n", template_name)
						).into_response()
					}
				}
			}
			MycFormatFamily::API => {
				view.get_api_response(&settings)
			}
		};
		// Set status code
		if response.status() == StatusCode::OK && status_code != StatusCode::OK {
			*response.status_mut() = status_code;
		}
		// Set cookies
		let cookie = Cookie::build("dns_resolver",settings.dns_resolver_id.to_string())
			.path("/")
			.same_site(cookie::SameSite::Strict)
			.finish();
		if let Ok(header_value) = HeaderValue::from_str(&cookie.to_string()) {
			response.headers_mut().append(
				SET_COOKIE,
				header_value,
			);
		}
		// return response
		response
	}
}
